package com.bwie.auth.service.impl;

import cn.hutool.core.util.RandomUtil;
import com.alibaba.fastjson.JSONObject;
import com.bwie.auth.feign.FeignService;
import com.bwie.auth.service.AuthService;
import com.bwie.common.constants.JwtConstants;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.UserRequest;
import com.bwie.common.domain.response.JwtResponse;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.UUID;
import java.util.concurrent.TimeUnit;
import java.util.regex.Pattern;

@Service
public class AuthServiceImpl implements AuthService {

    @Autowired
    private FeignService feignService;
    @Autowired
    private RedisTemplate<String,String> redisTemplate;
    @Autowired
    private HttpServletRequest request;

    @Override
    public Result sendCode(String userPhone) {
        if(!TvTel(userPhone)){
            return Result.error("手机号格式不正确");
        }
        if(StringUtils.isEmpty(userPhone)){
            return Result.error("手机号不能为空");
        }
        Result<User> byTel = feignService.findByTel(userPhone);
        User data = byTel.getData();
        if(data==null){
            return Result.error("账号不存在,请注册");
        }
        String code = "1"+RandomUtil.randomNumbers(5);
        redisTemplate.opsForValue().set(userPhone,code,5, TimeUnit.MINUTES);
        return Result.success(code);
    }

    @Override
    public Result<JwtResponse> login(UserRequest userRequest) {
        if(!TvTel(userRequest.getUserPhone())){
            return Result.error("手机号格式不正确");
        }
        if(StringUtils.isAllBlank(userRequest.getUserPhone(),userRequest.getCode())){
            return Result.error("手机号或验证码不能为空");
        }
        Result<User> byTel = feignService.findByTel(userRequest.getUserPhone());
        User data = byTel.getData();
        if(data==null){
            return Result.error("账号不存在,请注册");
        }
        String code = redisTemplate.opsForValue().get(userRequest.getUserPhone());
        if(!code.equals(userRequest.getCode())){
            return Result.error("验证码错误");
        }
        String s = UUID.randomUUID().toString().replaceAll("-", "");
        HashMap<String, Object> map = new HashMap<>();
        map.put(JwtConstants.USER_KEY,s);
        String token = JwtUtils.createToken(map);

        redisTemplate.opsForValue().set(TokenConstants.LOGIN_TOKEN_KEY+s, JSONObject.toJSONString(data),15,TimeUnit.MINUTES);

        JwtResponse jwtResponse = new JwtResponse();
        jwtResponse.setToken(token);
        jwtResponse.setExpireTime("15MIN");

        return Result.success(jwtResponse);
    }

    @Override
    public User userInfo() {
        String header = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(header);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        return JSONObject.parseObject(s,User.class);
    }

    private boolean TvTel(String userPhone) {
        Pattern compile = Pattern.compile("^(?:(?:\\+|00)86)?1(?:(?:3[\\d])|(?:4[5-7|9])|(?:5[0-3|5-9])|(?:6[5-7])|(?:7[0-8])|(?:8[\\d])|(?:9[1|8|9]))\\d{8}$");
        return compile.matcher(userPhone).matches();
    }
}








