package com.bwie.es.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.bwie.common.domain.Product;
import com.bwie.common.domain.request.ProductRequest;
import com.bwie.common.result.PageResult;
import com.bwie.common.result.Result;
import com.bwie.common.utils.StringUtils;
import com.bwie.es.service.EsService;
import org.apache.lucene.search.TotalHits;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.text.Text;
import org.elasticsearch.common.xcontent.XContentType;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightField;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
public class EsServiceImpl implements EsService {

    @Autowired
    private RestHighLevelClient restHighLevelClient;

    private final static String INDEX_NAME = "product";

    @Override
    public void batchAdd(List<Product> product) {
        try {
        BulkRequest bulkRequest = new BulkRequest();
        product.forEach(item->{
            bulkRequest.add(new IndexRequest(INDEX_NAME)
                    .id(item.getProductId()+"")
                    .source(JSONObject.toJSONString(item), XContentType.JSON));
        });
            restHighLevelClient.bulk(bulkRequest, RequestOptions.DEFAULT);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public Result<PageResult<Product>> search(ProductRequest productRequest) {
        long total=0;
        ArrayList<Product> list = new ArrayList<>();
        try {

            SearchRequest searchRequest = new SearchRequest(INDEX_NAME);

            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();

            BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();

            if(StringUtils.isNotEmpty(productRequest.getProductName())){
                boolQueryBuilder.must(QueryBuilders.matchQuery("productName",productRequest.getProductName()));
            }

            searchSourceBuilder.query(boolQueryBuilder);
            searchSourceBuilder.from((productRequest.getPageNum()-1)*productRequest.getPageSize());
            searchSourceBuilder.size(productRequest.getPageSize());


            searchSourceBuilder.highlighter(
                    new HighlightBuilder().field("productName")
                            .preTags("<font style =\"color:red;\">")
                            .postTags("</font>")
            );
            searchRequest.source(searchSourceBuilder);
            SearchResponse search = restHighLevelClient.search(searchRequest, RequestOptions.DEFAULT);
            SearchHits searchHits = search.getHits();
            total=searchHits.getTotalHits().value;
            SearchHit[] hits = searchHits.getHits();
            for (SearchHit hit : hits) {
                String sourceAsString = hit.getSourceAsString();
                Product product = JSONObject.parseObject(sourceAsString, Product.class);
                Map<String, HighlightField> highlightFields = hit.getHighlightFields();
                if(highlightFields!=null){
                    HighlightField highlightField = highlightFields.get("productName");
                    if(highlightField!=null){
                        Text[] fragments = highlightField.getFragments();
                        StringBuilder sb = new StringBuilder();
                        for (Text fragment : fragments) {
                            sb.append(fragment);
                        }
                        product.setProductName(sb.toString());
                    }
                }
                product.setProductId(Integer.valueOf(hit.getId()));
                list.add(product);
            }

        }catch (Exception e){
            e.printStackTrace();
        }
        return PageResult.toResult(total,list);
    }
}
